<?php

namespace Shipcloud\Controller;

use Shipcloud\Api;
use Shipcloud\Repository\ShipmentRepository;

class LabelController {
	const AJAX_UPDATE = 'wp_ajax_shipcloud_label_update';

	/**
	 * @var Api
	 */
	private $api;

	/**
	 * @var ShipmentRepository
	 */
	private $shipment_repository;

	/**
	 * Label_Controller constructor.
	 *
	 * @param Api                $api
	 * @param ShipmentRepository $shipment_repository
	 */
	public function __construct( Api $api, ShipmentRepository $shipment_repository ) {
		$this->api                 = $api;
		$this->shipment_repository = $shipment_repository;
	}

	public function update() {
		if ( ! $this->is_authenticated() ) {
			$this->error( 403, __( 'Not authenticated' ) );

			return;
		}

		if ( ! isset( $_REQUEST['shipment'] ) ) {
			$this->error( 400, __( 'Bad request' ) );

			return;
		}

		$data = $this->sanitize_request( $_REQUEST );

    $bank_name = $bank_code = $bank_account_holder = $bank_account_number = '';
    if (
      array_key_exists( 'additional_services', $data['shipment'] ) &&
      array_key_exists( 'cash_on_delivery', $data['shipment']['additional_services'] )
    ) {
      if (array_key_exists( 'bank_name', $data['shipment']['additional_services']['cash_on_delivery'] )) {
        $bank_name = $data['shipment']['additional_services']['cash_on_delivery']['bank_name'];
      }
      if (array_key_exists( 'bank_code', $data['shipment']['additional_services']['cash_on_delivery'] )) {
        $bank_code = $data['shipment']['additional_services']['cash_on_delivery']['bank_code'];
      }
      if (array_key_exists( 'bank_account_holder', $data['shipment']['additional_services']['cash_on_delivery'] )) {
        $bank_account_holder = $data['shipment']['additional_services']['cash_on_delivery']['bank_account_holder'];
      }
      if (array_key_exists( 'bank_account_number', $data['shipment']['additional_services']['cash_on_delivery'] )) {
        $bank_account_number = $data['shipment']['additional_services']['cash_on_delivery']['bank_account_number'];
      }
    }
    $bank_information = new \Shipcloud\Domain\ValueObject\BankInformation(
      $bank_name,
      $bank_code,
      $bank_account_holder,
      $bank_account_number
    );

        $pickup = \WC_Shipcloud_Order::handle_pickup_request($data);
        if (!empty($pickup)) {
            $data['shipment']['pickup'] = $pickup;
        }

    try {
      $order = $this->shipment_repository->findOrderByShipmentId( $data['shipment']['id'] );
      $sc_order = \WC_Shipcloud_Order::create_order( $order->get_id() );

      $data['shipment']['additional_services'] =
        $this->shipment_repository->additional_services_from_request(
            $data['shipment']['additional_services'],
            $data['shipment']['additional_services']['cash_on_delivery']['amount'],
            $data['shipment']['additional_services']['cash_on_delivery']['currency'],
            $bank_information,
            $data['shipment']['additional_services']['cash_on_delivery']['reference1'],
            $data['shipment']['carrier'],
            $sc_order
        );

            if (array_key_exists('customs_declaration', $data)) {
                $data['shipment']['customs_declaration'] = $this->handle_customs_declaration( $data['customs_declaration'] );
                unset($data['customs_declaration']);
            }

			$this->shipment_repository->update(
				$order,
				$data['shipment']
			);
		} catch ( \Exception $e ) {
			$this->error( 400, $e->getMessage() );

			return;
		}

		wp_send_json_success( $data['shipment'] );
	}

	protected function is_authenticated() {
		return is_admin() && is_user_logged_in();
	}

	protected function error( $httpCode, $errorMessage ) {
		if ( defined( 'DOING_AJAX' ) && DOING_AJAX ) {
			wp_send_json_error( new \WP_Error( $httpCode, $errorMessage ), $httpCode );
		}
	}

	/**
	 * @param array $data In case of null the current $_REQUEST will be used.
	 *
	 * @return mixed
	 */
	protected function sanitize_request( $data ) {
		unset( $data['action'] );
		unset( $data['shipment_order_id'] );

    // unset notification_email if it was not checked
    if (!isset($data['shipcloud_notification_email_checkbox'])) {
      unset( $data['shipcloud_notification_email'] );
    }
		// unset some unwanted input from additional services form
		// if (!isset($data['shipment']['additional_services']['delivery_time']['checked'])) {
		// 	unset($data['shipment']['additional_services']['delivery_time']['timeframe']);
		// }
		// if (!isset($data['shipment']['additional_services']['age_based_delivery']['checked'])) {
		// 	unset($data['shipment']['additional_services']['visual_age_check']['minimum_age']);
		// }
		// unset($data['shipment']['additional_services']['age_based_delivery']);

		return $data;
	}

    private function handle_customs_declaration( $data ) {
        if ('false' === $data['shown']) {
            return null;
        } else {
            $line_items = $data['items'];

            $items = array();
            foreach ( $line_items as $line_item_key => $line_item_data ) {
                $line_item_data['id'] = $line_item_key;
                array_push($items, $line_item_data);
            }

            $data['items'] = $items;
            return $data;
        }
    }
}
